<?php

namespace Drupal\commerce_stock_notifications\Form;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class NotificationForm.
 *
 * @package Drupal\commerce_stock_notifications\Form
 */
class NotificationForm extends FormBase {

  use StringTranslationTrait;

  /**
   * Current variation.
   *
   * @var \Drupal\commerce_product\Entity\ProductVariation
   */
  protected $selectedVariation = NULL;

  /**
   * Email validator.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  public static function create(ContainerInterface $container) {
    $form = parent::create($container);
    $form->messenger = $container->get('messenger');
    $form->configFactory = $container->get('config.factory');
    $form->emailValidator = $container->get('email.validator');
    $form->entityTypeManager = $container->get('entity_type.manager');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'commerce_stock_notifications_notification_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $selectedVariation = NULL) {
    $this->selectedVariation = $selectedVariation;
    $config = $this->configFactory->get('commerce_stock_notifications.config')->getRawData();

    return [
      'commerce_stock_notifications' => [
        '#type' => 'fieldset',
        '#title' => $config['oos_message'] ?? $this->t('Please notify me when this product is back in stock.'),
        'notify_email' => [
          '#type' => 'textfield',
          '#title' => $this->t('E-Mail'),
          '#required' => TRUE,
          '#default_value' => $this->currentUser()->getEmail(),
        ],
        'notify_submit' => [
          '#type' => 'submit',
          '#value' => $this->t('Notify me'),
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    // Check if user is logged in.
    if ($this->currentUser()->isAnonymous()) {
      $form_state->setError($form, $this->t('You must be authenticated to do that.'));
    }
    // Check if e-mail is valid.
    $email = $form_state->getValue('notify_email');
    if (!$this->emailValidator->isValid($email)) {
      $form_state->setErrorByName('notify_email', $this->t('Invalid e-mail'));
    }
    // Check if already subscribed.
    $notificationCount = $this->entityTypeManager
      ->getStorage('commerce_stock_notification')
      ->getQuery()
      ->condition('email', $email)
      ->condition('product_id', $this->selectedVariation->id())
      ->condition('sent_time', NULL, 'IS')
      ->count();

    if ($notificationCount->execute() > 0) {
      $duplicateMessage = AdminConfigForm::getValuesFromConfig()['duplicate_message'];
      $form_state->setErrorByName('notify_email', $duplicateMessage);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->entityTypeManager
      ->getStorage('commerce_stock_notification')
      ->create([
        'user_id' => $this->currentUser()->id(),
        'product_id' => $this->selectedVariation->id(),
        'submit_time' => (new DrupalDateTime())->getTimestamp(),
        'email' => $form_state->getValue('notify_email'),
      ])
      ->save();

    $this->messenger->addStatus(AdminConfigForm::getValuesFromConfig()['success_message']);
  }

}
